﻿using CRMPluginUtils;
using Microsoft.Win32;
using MicrosoftOutlookServiceProvider;
using System;
using System.Collections.Generic;
using System.IO;
using System.ServiceModel;

namespace MicrosoftOutlookPlugin
{
  public class OutlookHelper
  {
    private static ConfigurationManager configurationManager = new ConfigurationManager("3CXCRMUser.ini");
    private static List<ContactFolder> contactFolderList = null;
    private static ChannelFactory<IOutlookServiceProvider> channelFactory = null;

    private static void cloneContactFolderList(List<ContactFolder> remoteContactFolderList)
    {
      contactFolderList = new List<ContactFolder>();
      foreach (ContactFolder contactFolder in remoteContactFolderList)
        contactFolderList.Add(new ContactFolder(contactFolder.Path, contactFolder.EntryId, contactFolder.StoreId));
    }

    private static ContactFolder getContactFolder(string path)
    {
      if (contactFolderList != null)
      {
        foreach (ContactFolder contactFolder in contactFolderList)
        {
          if (contactFolder.Path == path) return contactFolder;
        }
      }
      return null;
    }

    private OutlookHelper()
    {
	
    }
	
	public static string GetVersion()
	{
	  try
	  {
			var path = AppDomain.CurrentDomain.BaseDirectory + @"DotNetScripts\Outlook\VERSION";
			LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", path);
			return System.IO.File.ReadAllText(path);
		}
	  catch (Exception e)
	  {
			LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "VERSION not found");
	  }

		return string.Empty;
	}

	

    public static IOutlookServiceProvider GetServiceProvider()
    {
      if (channelFactory == null)
      {
        NetNamedPipeBinding binding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
        string clientEndpointAddress = "net.pipe://localhost/OutlookServiceProvider_" + OwnerFinder.GetCurrentProcessOwnerId();
        EndpointAddress address = new EndpointAddress(clientEndpointAddress);
        LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Creating client channel factory to connect to Outlook add-in to pop-up contact records using address: " + clientEndpointAddress);
        channelFactory = new ChannelFactory<IOutlookServiceProvider>(binding, address);
      }
      return channelFactory.CreateChannel();
    }

    public static void RefreshContactFolderList(IOutlookServiceProvider serviceProvider)
    {
      cloneContactFolderList(serviceProvider.GetContactFolderList());
    }

    public static List<ContactFolder> GetContactFolderList()
    {
      return contactFolderList;
    }

    public static List<ContactFolder> GetEnabledContactFolderList()
    {
      List<ContactFolder> enabledContactFolderList = new List<ContactFolder>();
      bool isEveryContactFolderEnabled = configurationManager.GetValue("Microsoft Outlook Plugin", "IsEveryContactFolderEnabled", "True") == "True";
      if (isEveryContactFolderEnabled)
      {
        foreach (ContactFolder contactFolder in contactFolderList)
          enabledContactFolderList.Add(contactFolder);
      }
      else
      {
        int enabledContactFolderCount = Convert.ToInt32(configurationManager.GetValue("Microsoft Outlook Plugin", "EnabledContactFolderCount", "0"));
        for (int index = 0; index < enabledContactFolderCount; ++index)
        {
          string contactFolderPath = configurationManager.GetValue("Microsoft Outlook Plugin", "EnabledContactFolder_" + index.ToString(), "");
          ContactFolder contactFolder = getContactFolder(contactFolderPath);
          if (contactFolder != null) enabledContactFolderList.Add(contactFolder);
        }
      }

      return enabledContactFolderList;
    }
    
    public static void RegisterAddin()
    {
        var ver = GetVersion();
        if (!string.IsNullOrEmpty(ver))
            LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Plugin Version: " + ver);

      RegistryKey localMachineRegistryKey = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry32);
      RegistryKey txcPhoneRegistryKey = localMachineRegistryKey.OpenSubKey(@"SOFTWARE\3CX\3CXPhone for Windows", false);
      if (txcPhoneRegistryKey == null) throw new ApplicationException("3CXPhone for Windows could not be detected.");
      string tcxPhonePath = txcPhoneRegistryKey.GetValue("Path") as string;

      LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Creating Outlook add-in registry entries...");
      RegistryKey currentUserRegistryKey = RegistryKey.OpenBaseKey(RegistryHive.CurrentUser, RegistryView.Registry32);
      RegistryKey softwareRegistryKey = currentUserRegistryKey.CreateSubKey("Software", RegistryKeyPermissionCheck.ReadWriteSubTree);
      RegistryKey softwareMicrosoftRegistryKey = softwareRegistryKey.CreateSubKey("Microsoft");
      RegistryKey softwareMicrosoftOfficeRegistryKey = softwareMicrosoftRegistryKey.CreateSubKey("Office");
      RegistryKey softwareMicrosoftOfficeOutlookRegistryKey = softwareMicrosoftOfficeRegistryKey.CreateSubKey("Outlook");
      RegistryKey softwareMicrosoftOfficeOutlookAddinsRegistryKey = softwareMicrosoftOfficeOutlookRegistryKey.CreateSubKey("Addins");
      RegistryKey softwareMicrosoftOfficeOutlookAddinsAddInRegistryKey = softwareMicrosoftOfficeOutlookAddinsRegistryKey.CreateSubKey("TCX.MicrosoftOutlookAddin");
      softwareMicrosoftOfficeOutlookAddinsAddInRegistryKey.SetValue("Description", "Allows looking for contact information in Contact folders, and launch calls to contacts using 3CX with a simple click", RegistryValueKind.String);
      softwareMicrosoftOfficeOutlookAddinsAddInRegistryKey.SetValue("FriendlyName", "3CX Outlook Add-in", RegistryValueKind.String);
      softwareMicrosoftOfficeOutlookAddinsAddInRegistryKey.SetValue("LoadBehavior", 3, RegistryValueKind.DWord);
      softwareMicrosoftOfficeOutlookAddinsAddInRegistryKey.SetValue("Manifest", Path.Combine(tcxPhonePath, "DotNetScripts", "Outlook", "References", "3CXOutlookAddin.vsto") + "|vstolocal", RegistryValueKind.String);
      LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Outlook add-in registry entries successfully created.");
    }
  }
}
